// Copyright 2022 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package pathfinder

import (
	"github.com/stretchr/testify/assert"
	"testing"
)

func TestGetIndexFromArray(t *testing.T) {
	m := map[string]any{
		"A": "true",
		"B": []string{
			"1",
			"2",
			"3",
		},
	}

	v, err := GetValue[map[string]any](m, "B.1")
	if err != nil {
		t.Error(err)
	}

	assert.Equal(t, "2", v)

}

func TestGetValueFrom(t *testing.T) {

	m := map[string]string{
		"KeyA": "true",
		"KeyB": "2",
		"KeyC": "3",
		"KeyD": "4.0",
	}

	v, err := GetValue[map[string]string](m, "KeyA")
	if err != nil {
		t.Error(err)
	}

	assert.Equal(t, v, "true")

}

func TestPathGetValueFromX(t *testing.T) {
	m := map[string]any{
		"A": "true",
		"B": map[string]any{
			"C": 2,
			"D": map[string]any{
				"E": "3",
			},
		},
		"X": "3",
		"Y": "4.0",
	}

	v, err := GetValue[map[string]any](m, "B.D.E")
	if err != nil {
		t.Error(err)
	}

	assert.Equal(t, "3", v)

	v, err = GetValue[map[string]any](m, "B.C")
	if err != nil {
		t.Error(err)
	}

	assert.Equal(t, 2, v)

}

func TestPathGetValueFrom(t *testing.T) {

	type PathfindTestStruct1 struct {
		A    bool
		Sub1 struct {
			B    int
			Sub2 struct {
				C bool
			}
			D int
		}
	}

	var testData PathfindTestStruct1
	testData.A = true
	testData.Sub1.B = 2
	testData.Sub1.Sub2.C = true
	testData.Sub1.D = 4

	GetValue[PathfindTestStruct1](testData, "Sub1.B")
	GetValue[PathfindTestStruct1](testData, "Sub1.Sub2.C")
	GetValue[PathfindTestStruct1](testData, "Sub1.D")

}
